==> diffusion-3.0.3.tar.gz: diffusion-3.0.3/apps/sysapps/tag.cc <==
//
// tag.cc         : Tag Application
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: tag.cc,v 1.8 2001/06/29 21:48:13 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "tag.hh"

TagApp *app;

void MyFilterReceive::recv(Message *msg, handle h)
{
  app->recv(msg, h);
}

void TagApp::recv(Message *msg, handle h)
{
  if (h != filterHandle){
    fprintf(stderr, "Error: TagApp::recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
    return;
  }

  ProcessMessage(msg);

  ((DiffusionRouting *)dr)->sendMessageToNext(msg, h);
}

void TagApp::ProcessMessage(Message *msg)
{
  char *original_route;
  char *new_route;
  int len, total_len;
  NRSimpleAttribute<char *> *route = NULL;

  // Can't do anything if node id unknown
  if (!id)
    return;

  route = RouteAttr.find(msg->msg_attr_vec);
  if (!route){
    fprintf(stderr, "Error: Can't find the route attribute !\n");
    return;
  }

  original_route = route->getVal();
  len = strlen(original_route);

  if (len == 0){
    total_len = strlen(id);
    // Route is empty, need to allocate memory
    // for our id and the terminating '\0'
    new_route = new char[(total_len + 1)];
    strcpy(new_route, id);
    if (new_route[total_len] != '\0'){
      fprintf(stderr, "Warning: String must end with NULL !\n");
    }
  }
  else{
    // Route already exists. We need to allocate
    // memory for the current route + ':' + our
    // id + the terminating '\0'
    total_len = len + strlen(id) + 1;
    new_route = new char[(total_len + 1)];
    strcpy(new_route, original_route);
    new_route[len] = ':';
    strcpy(&new_route[len+1], id);
    if (new_route[total_len] != '\0'){
      fprintf(stderr, "Warning: String must end with NULL !\n");
    }
  }

  // Debug
  fprintf(stderr, "Tag App: Original route : %s\n", original_route);
  fprintf(stderr, "Tag App: New route : %s\n", new_route);

  route->setVal(new_route);

  // Free memory
  delete [] new_route;
}

handle TagApp::setupFilter()
{
  NRAttrVec attrs;
  handle h;

  // This is a dummy attribute for filtering
  // It matches everything
  attrs.push_back(RouteAttr.make(NRAttribute::EQ_ANY, ""));

  h = ((DiffusionRouting *)dr)->addFilter(&attrs, TAG_APP_PRIORITY, fcb);

  ClearAttrs(&attrs);
  return h;
}

void TagApp::run()
{
  // Doesn't do anything
  while (1){
    sleep(10);
  }
}

void TagApp::getNodeId()
{
  fprintf(stderr, "getNodeID function not yet implemented !\n");
  exit(-1);
  // Future implementation for the inter-module API
}

void TagApp::usage()
{
  fprintf(stderr, "Usage: tag [-p port] [-h]\n\n");
  fprintf(stderr, "\t-p - Uses port 'port' to talk to diffusion\n");
  fprintf(stderr, "\t-h - Prints this information\n");
  fprintf(stderr, "\n");
  exit(0);
}

TagApp::TagApp(int argc, char **argv)
{
  char *id_env = NULL;
  char buffer[BUFFER_SIZE];
  int flag;
  int opt;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  // Parse command line options
  opterr = 0;

  while (1){
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	fprintf(stderr, "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      fprintf(stderr, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      fprintf(stderr, "Parameter missing !\n");
      usage();

      break;

    }

    if (opt == -1)
      break;
  }

  id = NULL;
  node_id = 0;

  // Create Diffusion Routing class
  dr = NR::createNR(diffusion_port);
  fcb = new MyFilterReceive;

  // Try to figure out the node ID
  id_env = getenv("scadds_addr");
  if (id_env){
    node_id = atoi(id_env);
    flag = snprintf(&buffer[0], BUFFER_SIZE, "%d", node_id);
    if (flag == -1 || flag == BUFFER_SIZE){
      fprintf(stderr, "Error: Buffer too small !\n");
      exit(-1);
    }
    id = strdup(&buffer[0]);
  }
  else{
    getNodeId();
  }
  // Set up the filter
  filterHandle = setupFilter();
  fprintf(stderr, "Tag App subscribed to *, received handle %d\n", filterHandle);
  fprintf(stderr, "Tag App initialized !\n");
}

int main(int argc, char **argv)
{
  // Initialize and run the Tag App
  app = new TagApp(argc, argv);
  app->run();

  return 0;
}

==> diffusion-3.0.4.tar.gz: diffusion-3.0.4/apps/sysapps/tag.cc <==
//
// tag.cc         : Tag Filter
// author         : Fabio Silva
//
// Copyright (C) 2000-2001 by the Unversity of Southern California
// $Id: tag.cc,v 1.10 2001/09/07 23:10:29 fabio Exp $
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License,
// version 2, as published by the Free Software Foundation.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
//
//

#include "tag.hh"

TagFilter *app;

void MyFilterReceive::recv(Message *msg, handle h)
{
  app->recv(msg, h);
}

void TagFilter::recv(Message *msg, handle h)
{
  if (h != filterHandle){
    fprintf(stderr, "Error: TagFilter::recv received message for handle %d when subscribing to handle %d !\n", h, filterHandle);
    return;
  }

  ProcessMessage(msg);

  ((DiffusionRouting *)dr)->sendMessageToNext(msg, h);
}

void TagFilter::ProcessMessage(Message *msg)
{
  char *original_route;
  char *new_route;
  int len, total_len;
  NRSimpleAttribute<char *> *route = NULL;

  // Can't do anything if node id unknown
  if (!id)
    return;

  route = RouteAttr.find(msg->msg_attr_vec);
  if (!route){
    fprintf(stderr, "Error: Can't find the route attribute !\n");
    return;
  }

  original_route = route->getVal();
  len = strlen(original_route);

  if (len == 0){
    total_len = strlen(id);
    // Route is empty, need to allocate memory
    // for our id and the terminating '\0'
    new_route = new char[(total_len + 1)];
    strcpy(new_route, id);
    if (new_route[total_len] != '\0')
      fprintf(stderr, "Warning: String must end with NULL !\n");
  }
  else{
    // Route already exists. We need to allocate
    // memory for the current route + ':' + our
    // id + the terminating '\0'
    total_len = len + strlen(id) + 1;
    new_route = new char[(total_len + 1)];
    strcpy(new_route, original_route);
    new_route[len] = ':';
    strcpy(&new_route[len+1], id);
    if (new_route[total_len] != '\0'){
      fprintf(stderr, "Warning: String must end with NULL !\n");
    }
  }

  // Debug
  fprintf(stderr, "Tag Filter: Original route : %s\n", original_route);
  fprintf(stderr, "Tag Filter: New route : %s\n", new_route);

  route->setVal(new_route);

  // Free memory
  delete [] new_route;
}

handle TagFilter::setupFilter()
{
  NRAttrVec attrs;
  handle h;

  // Match all packets with a Route Attribute
  attrs.push_back(RouteAttr.make(NRAttribute::EQ_ANY, ""));

  h = ((DiffusionRouting *)dr)->addFilter(&attrs, TAG_FILTER_PRIORITY, fcb);

  ClearAttrs(&attrs);
  return h;
}

void TagFilter::run()
{
  // Doesn't do anything
  while (1){
    sleep(10);
  }
}

void TagFilter::getNodeId()
{
  fprintf(stderr, "getNodeID function not yet implemented !\n");
  exit(-1);
  // Future implementation for the inter-module API
}

void TagFilter::usage()
{
  fprintf(stderr, "Usage: tag [-p port] [-h]\n\n");
  fprintf(stderr, "\t-p - Uses port 'port' to talk to diffusion\n");
  fprintf(stderr, "\t-h - Prints this information\n");
  fprintf(stderr, "\n");
  exit(0);
}

TagFilter::TagFilter(int argc, char **argv)
{
  char *id_env = NULL;
  char buffer[BUFFER_SIZE];
  int flag;
  int opt;
  u_int16_t diffusion_port = DEFAULT_DIFFUSION_PORT;

  // Parse command line options
  opterr = 0;

  while (1){
    opt = getopt(argc, argv, "hp:");
    switch (opt){

    case 'p':

      diffusion_port = (u_int16_t) atoi(optarg);
      if ((diffusion_port < 1024) || (diffusion_port >= 65535)){
	fprintf(stderr, "Error: Diffusion port must be between 1024 and 65535 !\n");
	exit(-1);
      }

      break;

    case 'h':

      usage();

      break;

    case '?':

      fprintf(stderr, "Error: %c isn't a valid option or its parameter is missing !\n", optopt);
      usage();

      break;

    case ':':

      fprintf(stderr, "Parameter missing !\n");
      usage();

      break;

    }

    if (opt == -1)
      break;
  }

  id = NULL;
  node_id = 0;

  // Create Diffusion Routing class
  dr = NR::createNR(diffusion_port);
  fcb = new MyFilterReceive;

  // Try to figure out the node ID
  id_env = getenv("scadds_addr");
  if (id_env){
    node_id = atoi(id_env);
    flag = snprintf(&buffer[0], BUFFER_SIZE, "%d", node_id);
    if (flag == -1 || flag == BUFFER_SIZE){
      fprintf(stderr, "Error: Buffer too small !\n");
      exit(-1);
    }
    id = strdup(&buffer[0]);
  }
  else{
    getNodeId();
  }
  // Set up the filter
  filterHandle = setupFilter();
  fprintf(stderr, "Tag Filter subscribed to *, received handle %d\n", filterHandle);
  fprintf(stderr, "Tag Filter initialized !\n");
}

int main(int argc, char **argv)
{
  // Initialize and run the Tag Filter
  app = new TagFilter(argc, argv);
  app->run();

  return 0;
}
